#####
#
# Convert SIMO simulated data to CSV files
# 2020-02-21
#
#####

# rm(list = ls())  # remove all files in memory if needed


library(dplyr)


### Set the working path to this R-project
path <- paste0(getwd())



### Define the input & output folders
# ------------------------------------

#   Input: contains all .db and will convert them to 
#   to .csv in 'output' folder
inputFolder  <- paste(path, "input_data", sep = "/")
outputFolder <- paste(path, "output",     sep = "/")



### The corresponding db files need to be stored in folder "input_data"
### The output .csv files will be stored in folder "output" 

### JUST notes:

### specify which one should be read:


# FOR MSC STUDENTS - total watershed have been already simulated:
# variant simulated with WIN version of NON-climate sensitive SIMO version
#                    "without"     no climate change
#                    "without_SA"  no climate change and set aside without deadwood extraction (no other management regimes!)
# 
# variants simulated with WIN version of climate sensitive SIMO version, climate model HadGEM2
#                    "CC45"        climate change with RCP scenraio 4.5 
#                    "CC45_SA"     RCP 4.5 and set aside without deadwood extraction (no other management regimes!)
#                    "CC85"        climate change with RCP scenario 8.5 
#                    "CC85_SA"     RCP 8.5 and set aside without deadwood extraction (no other management regimes!)


# FOR COMPARISON OF CLIMATE MODEL AND SIMO VERSIONS - ONLY 5 stands per watershed so far simulated
# variants simulated with SIMO Linux version on cPouta, indicated by "p"
# with climate model HadGEM2
#                   "CC45_p"      RCP scenario 4.5
#                   "CC85_p"      RCP scenario 8.5
#                   "CC26_p"      NEW: RCP scenario 2.6
#                   "CC0_p        NEW: RCP scenario 0, represents NO CLIMATE CHANGE ?? 
#
# with climate model CanESM2 (like in FutureBioEcon), indicated further by "canesm"
#                   "CC45_p_canesm"      RCP sceanrio 4.5
#                   "CC85_p_canesm"      RCP scenario 8.5
#                   "CC26_p_canesm"      NEW: RCP scenario 2.6
#                   "CC0_p_canesm"       NEW: RCP scenario 0


# -------------------
# Input db :
# -------------------


# List all .db in 'input'
db_input <- list.files(inputFolder,
                       pattern = ".db$")


#Remove the ending ".db" from the list to create a database name
db_names <- gsub(".db", "", db_input)



# Identify columns from UNIT table to export to .cvs
# -----------------------------------------
# different columns are stored in stratum or in comp_unit
# if more variables are required, need to update the SQL 
# created script 'structure_SIMO_rslDB.R'
# An overview on all available SIMO outcomes can be found under: params/Overview_outcomes_SIMO.xlsx

columns <-  paste0("id,
                   year,
                   branch,
                   branch_desc,
                   branching_group,
                   Age,
                   area,
                   cash_flow,
                   V_total_deadwood,
                   BA,
                   V,
                   SC,
                   N,
                   H_dom,
                   D_gm,
                   Harvested_V,
                   Biomass,
                   income_biomass,
                   CARBON_STORAGE, 
                   MAIN_SP")



# Try to add THIN value to SA_scenario???
# 

# --------------------
#   Process: QUERY
# --------------------

# Query each file in the folder depending the 
# of SA management 
# ad\nd create UNIT table (stored in specific .db)

for (name in db_names) {
 
    # CHeck if the name contains "_SA" regime
  # if YES - run SA query
  # if NO  - run normal query
  
  
  if(grepl("_SA", name)){
    
     # This needs specific SQL, since the database does not contain harvest information
     source(paste0(path, "/structure_SIMO_rslDB_SA.R"))
     
   } else {
     
     # Run the script with the SQL query for all management regimes besides SA
     source(paste0(path, "/structure_SIMO_rslDB_FBE.R")) # Query 
     
   }
 }
 

# ---------------------
# Process: Export .csv
# ---------------------


# Create the final CSV for each database
# from UNIT table 

source(paste(path, "loadDB.R", sep = "/")) 









